import xml.etree.ElementTree as ElementTree
from typing import Union, Optional
from django.template import Library
from hrenpack import IntStr
from hrenpack.filework.xml import XMLParser
from rusgram import pluralize

register = Library()


@register.filter
def get_item(dictionary, key, default=None):
    return dictionary.get(key, default)


@register.filter
def svg_to_html(value):
    svg = XMLParser(value).find('svg')
    return str(svg)


@register.filter
def addition(value, arg):
    return value + arg


@register.filter
def as_int(value):
    return int(value)


"""
@register.filter
def word_format(value: Union[IntStr, float], word: str) -> str:
    if isinstance(value, str):
        value = int(value)
    elif isinstance(value, float):
        return f'{value} {word}ов'
    else:
        match str(value)[-1]:
            case '1':
                return f'{value} {word}'
            case '2' | '3' | '4':
                return f'{value} {word}а'
            case _:
                return f'{value} {word}ов'
"""


@register.filter
def word_format(value: int, word: str) -> str:
    return pluralize(value, word)


@register.filter
def true_false(value):
    return str(value).lower()



import os
from datetime import datetime
from pathlib import Path
from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand
from hrenpack.classes import TransposedList
from hrenpack.listwork import dict_get
from hrendjango.constants import LANGS_PLURALIZE


class Command(BaseCommand):
    help = 'Генерирует PO-файлы для указанного приложения и языков'

    def add_arguments(self, parser):
        parser.add_argument('app_name', type=str, help='Название приложения')
        parser.add_argument('langs', nargs='*', type=str, help='Коды языков')
        parser.add_argument('--exclude-date', '-d', action='store_true',
                            help='Исключить даты из заголовков PO-файлов')
        parser.add_argument('--all-langs', '-a', action='store_true', help="Использовать все языки, поддерживаемые Django")

    def handle(self, *args, **options):
        app_name = options['app_name']
        date = str(datetime.now())[:16]
        exclude_date = options['exclude_date']
        verbose = options['verbosity'] > 1
        langs = options['langs'] if not options['all_langs'] else list(LANGS_PLURALIZE)
        langs = TransposedList(settings.LANGUAGES)[0] if not langs else langs

        if verbose:
            self.stdout.write(f"Начало генерации PO для приложения: {app_name}")
            self.stdout.write(f"Языки: {langs}")

        try:
            app_config = apps.get_app_config(app_name)
            app_dir = app_config.path
            if verbose:
                self.stdout.write(f"Приложение найдено в: {app_dir}")
        except LookupError:
            self.stderr.write(f'Приложение "{app_name}" не найдено')
            return

        po_props_dict = {
            "Project-Id-Version": app_name,
            "POT-Creation-Date": date,
            "PO-Revision-Date": date,
            "MIME-Version": "1.0",
            "Content-Type": "text/plain; charset=UTF-8",
            "Content-Transfer-Encoding": "8bit",
        }

        for lang in langs:
            if lang not in LANGS_PLURALIZE:
                self.stderr.write(f'Язык "{lang}" не поддерживается, пропускаем')
                continue

            if verbose:
                self.stdout.write(f"\nОбработка языка: {lang}")

            # Подготовка заголовков PO
            po_text = 'msgid ""\nmsgstr ""\n'
            dct = po_props_dict.copy()
            dct.update({
                'Language': lang,
                'Plural-Forms': LANGS_PLURALIZE[lang]
            })

            if exclude_date:
                dct.pop('POT-Creation-Date', None)
                dct.pop('PO-Revision-Date', None)

            for key, value in dct.items():
                po_text += f'"{key}: {value}\\n"\n'

            # Создание директории, если её нет
            po_dir = Path(app_dir) / 'locale' / lang / 'LC_MESSAGES'
            if verbose:
                self.stdout.write(f"Целевая директория: {po_dir}")

            try:
                po_dir.mkdir(parents=True, exist_ok=True)
                if verbose:
                    self.stdout.write(f"Директория создана/существует: {po_dir.exists()}")
            except Exception as e:
                self.stderr.write(f'Ошибка создания директории {po_dir}: {e}')
                continue

            po_file = po_dir / 'django.po'
            if verbose:
                self.stdout.write(f"Попытка записи в: {po_file}")

            try:
                with po_file.open('w', encoding='utf-8') as file:
                    file.write(po_text)
                self.stdout.write(f'Файл {po_file} успешно создан')
            except IOError as e:
                self.stderr.write(f'Ошибка записи в файл {po_file}: {e}')
                # Вывод полного пути для отладки
                self.stderr.write(f'Полный путь: {po_file.absolute()}')



from hrendjango.utils import get_model

__all__ = ['get_file_model', 'get_image_model']


def get_file_model():
    return get_model('HRENDJANGO_FILE_MODEL')


def get_image_model():
    return get_model('HRENDJANGO_IMAGE_MODEL')


import uuid
from django.db import models
from .mixins import DeleteFilesMixin

__all__ = ['AbstractFile', 'AbstractImageFile']


class AbstractFile(DeleteFilesMixin, models.Model):
    file = models.FileField(upload_to='files/', verbose_name="Файл")
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name="Уникальный идентификатор")
    date_create = models.DateTimeField(auto_now_add=True, verbose_name="Дата создания")
    date_update = models.DateTimeField(auto_now=True, verbose_name="Дата изменения")
    filename = models.CharField(max_length=255, verbose_name="Имя файла")

    class Meta:
        abstract = True
        verbose_name = "файл"
        verbose_name_plural = "Файлы"

    def __str__(self):
        return self.filename


class AbstractImageFile(AbstractFile):
    file = models.ImageField(upload_to='images/', verbose_name="Изображение")

    class Meta:
        abstract = True
        verbose_name = "изображение"
        verbose_name_plural = "Изображения"



from django.conf import settings
from django.http import Http404
from django.shortcuts import render
from hrenpack.framework.django.views import *


def robots_txt(request):
    path = settings.BASE_DIR / 'robots.txt'
    if not path.is_file():
        raise Http404





from django.core.files.uploadedfile import UploadedFile
from django.utils.safestring import mark_safe
from hrenpack.framework.django.forms import *
from .models import get_file_model, get_image_model


class UUIDInput(forms.TextInput):
    template_name = 'hrendjango/widgets/UUIDInput.html'

    def __init__(self, attrs=None, uuid_name: str = 'uuid'):
        super(UUIDInput, self).__init__(attrs)
        self.uuid_name = uuid_name

    def get_context(self, name, value, attrs=None):
        context = super(UUIDInput, self).get_context(name, value, attrs)
        context['input__style'] = '''
            text-align: center;
            cursor: pointer;
        '''
        context['name'] = self.uuid_name
        return context


class UUIDField(forms.UUIDField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('widget', UUIDInput(uuid_name=kwargs.pop('uuid_name')))
        super(UUIDField, self).__init__(*args, **kwargs)


class ClearableFileInput(forms.ClearableFileInput):
    template_name = 'hrendjango/widgets/ClearableFileInput.html'

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['clear__style'] = """
            display: inline-flex;
            align-items: center;
            margin-top: 0;
            margin-left: 40%;
            margin-bottom: 10px;
        """
        return context


class MultipleClearableFileInput(ClearableFileInput):
    allow_multiple_selected = True


class MultipleFileField(forms.FileField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('widget', MultipleClearableFileInput())
        super().__init__(*args, **kwargs)

    def clean(self, data, initial=None):
        if isinstance(data, (list, tuple)):
            return [super().clean(item, initial) for item in data]
        return super().clean(data, initial)


class FileOrImageField(MultipleFileField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('file_model', get_file_model())
        kwargs.setdefault('image_model', get_image_model())
        self.FileModel = kwargs.pop('file_model')
        self.ImageModel = kwargs.pop('image_model')
        super().__init__(*args, **kwargs)

    def save(self, data, initial=None):
        # Вызываем родительский метод clean для базовой валидации
        files = self.clean(data, initial)

        # Обрабатываем каждый файл
        saved_files = []
        for file in files:
            if isinstance(file, UploadedFile):
                if file.content_type.startswith('image'):
                    image_file = self.ImageModel(file=file, filename=file.name)
                    image_file.save()
                    saved_files.append(image_file)
                else:
                    other_file = self.FileModel(file=file, filename=file.name)
                    other_file.save()
                    saved_files.append(other_file)

        # Возвращаем список сохраненных файлов
        return saved_files

    def clean(self, data, initial=None):
        files = super().clean(data, initial)
        return files



from django import forms
from django.conf import settings
from django.core.exceptions import FieldDoesNotExist
from hrenpack.framework.django.mixins import *
from .models import get_image_model
from .decorators import debug_required


class SaveFilesMixin(forms.ModelForm):
    file_or_image_field_name: str = 'files'

    def save(self, commit=True):
        instance = super().save(False)
        if commit:
            instance.save()

            # Обработка загруженных файлов
        uploaded_files = self.cleaned_data.get(self.file_or_image_field_name)
        if uploaded_files:
            saved_files = self.fields[self.file_or_image_field_name].save_files(uploaded_files)
            # Привязываем сохраненные файлы к странице
            for file in saved_files:
                if isinstance(file, get_image_model()):
                    instance.images.add(file)
                else:
                    instance.files.add(file)
            instance.save()

        return instance


class VerboseNameLabelMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Перебираем все поля формы
        for field_name, field in self.fields.items():
            if hasattr(self.Meta, 'labels') and field_name in self.Meta.labels:
                continue
            # Если поле связано с моделью и имеет verbose_name, используем его как label
            try:
                if hasattr(self.Meta, 'model'):
                    model_field = self.Meta.model._meta.get_field(field_name)
                    if hasattr(model_field, 'verbose_name'):
                        field.label = model_field.verbose_name
            except FieldDoesNotExist:
                continue


class RobotOffMixin:
    noindex: bool = True
    nofollow: bool = False

    def dispatch(self, request, *args, **kwargs):
        response = super().dispatch(request, *args, **kwargs)
        headers = list()
        if self.noindex:
            headers.append('noindex')
        if self.nofollow:
            headers.append('nofollow')
        response.headers['X-Robots-Tag'] = ', '.join(headers)
        return response


class DebugRequiredMixin:
    @debug_required
    def __new__(cls, *args, **kwargs):
        return super().__new__(cls)



from django.template import Library

register = Library()


@register.simple_tag(name='print')
def print_(*values, sep: str = ' ', end: str = '\n'):
    print(*values, sep=sep, end=end)
    return ''



from django.template import Library
from django.template.base import Node, TemplateSyntaxError

register = Library()


class ConditionBlockNode(Node):
    def __init__(self, conditions, nodelist_else=None, variable_name=None):
        self.conditions = conditions  # список кортежей (условие, nodelist)
        self.nodelist_else = nodelist_else
        self.variable_name = variable_name

    def render(self, context):
        # Проверяем условия по порядку
        for condition, nodelist in self.conditions:
            if condition.eval(context):
                result = nodelist.render(context)
                if self.variable_name:
                    context[self.variable_name] = result
                    return ''
                return result

        # Если ни одно условие не выполнилось, используем else
        if self.nodelist_else:
            result = self.nodelist_else.render(context)
            if self.variable_name:
                context[self.variable_name] = result
                return ''
            return result

        if self.variable_name:
            context[self.variable_name] = ''
            return ''

        return ''


@register.tag(name='conditionblock')
def do_conditionblock(parser, token):
    """
    Синтаксис:
    {% conditionblock condition as var_name %}
        содержимое при истинном условии
    {% elif condition %}
        содержимое при истинном elif
    {% else %}
        содержимое при всех ложных условиях
    {% endconditionblock %}
    """
    bits = token.split_contents()
    variable_name = None

    # Обрабатываем часть 'as variable_name'
    if len(bits) > 2 and bits[-2] == 'as':
        variable_name = bits[-1]
        condition = parser.compile_filter(' '.join(bits[1:-2]))
    elif len(bits) > 1:
        condition = parser.compile_filter(' '.join(bits[1:]))
    else:
        raise TemplateSyntaxError("'conditionblock' требует хотя бы одного аргумента")

    conditions = [(condition, parser.parse(('elif', 'else', 'endconditionblock')))]

    token = parser.next_token()

    # Обрабатываем все elif
    while token.contents.startswith('elif'):
        condition = parser.compile_filter(token.contents[4:].strip())
        nodelist = parser.parse(('elif', 'else', 'endconditionblock'))
        conditions.append((condition, nodelist))
        token = parser.next_token()

    # Обрабатываем else, если есть
    nodelist_else = None
    if token.contents == 'else':
        nodelist_else = parser.parse(('endconditionblock',))
        token = parser.next_token()

    if token.contents != 'endconditionblock':
        raise TemplateSyntaxError("Ожидается закрывающий тег 'endconditionblock'")

    return ConditionBlockNode(conditions, nodelist_else, variable_name)


class VarBlockNode(Node):
    def __init__(self, nodelist, variable_name=None):
        self.nodelist = nodelist
        self.variable_name = variable_name

    def render(self, context):
        content = self.nodelist.render(context)

        # Если указано имя переменной, сохраняем результат в контекст
        if self.variable_name:
            context[self.variable_name] = content
            return ''

        return content


@register.tag(name='varblock')
def do_varblock(parser, token):
    """
        Тег с синтаксисом:
        {% tag as variable_name %}
            содержимое
        {% endtag %}
        """
    bits = token.split_contents()
    variable_name = None

    if len(bits) > 1:
        if bits[1] != 'as':
            raise TemplateSyntaxError("Ожидается 'as' после имени тега")
        if len(bits) < 3:
            raise TemplateSyntaxError("Необходимо указать имя переменной после 'as'")
        variable_name = bits[2]

    nodelist = parser.parse(('endtag',))
    parser.delete_first_token()

    return VarBlockNode(nodelist, variable_name)



from importlib import import_module
from django.contrib.auth import get_user_model
from django.utils.functional import SimpleLazyObject
from hrenpack.framework.django import *
from django.apps import apps
from django.conf import settings
from django.db import migrations
from django.core.exceptions import ImproperlyConfigured
from . import translator


def create_model(
    name: Any,
    fields: Any,
    options: Any = None,
    bases: Any = None,
    managers: Any = None
):
    models = settings.HRENDJANGO_ACTIVE_MODELS
    if name in models or models == 'all':
        return migrations.CreateModel(name, fields, options, bases, managers)


def get_model(setting_name: str):
    setting_name = setting_name.upper()
    setting = getattr(settings, setting_name)
    try:
        return apps.get_model(setting, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(f"{setting_name} must be of the form 'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(f"{setting_name} refers to model '%s' that has not been installed" % setting)


def get_project_name():
    return settings.ROOT_URLCONF.split('.')[0]


def safe_get_user_model():
    try:
        return get_user_model()
    except ImproperlyConfigured:
        return settings.AUTH_USER_MODEL


def translate_text(text, output_lang: [Optional[str]] = None, input_lang: str = 'auto'):
    if output_lang is None:
        output_lang = settings.LANGUAGE_CODE.split('-')[0]
    return translator.translate(text, src=input_lang, dest=output_lang).text



from django.template import Library
from django.templatetags.static import static

register = Library()
STYLES = {'hrendjango.css'}
SCRIPTS = {'hrendjango.js'}


@register.inclusion_tag('hrendjango/includes/init_styles.html')
def hrendjango_init_styles():
    return dict(styles=[static('hrendjango/css/' + css) for css in STYLES])


@register.inclusion_tag('hrendjango/includes/init_scripts.html')
def hrendjango_init_scripts():
    return dict(scripts=[static('hrendjango/js/' + js) for js in SCRIPTS])


@register.inclusion_tag('hrendjango/includes/init.html')
def hrendjango_init(styles=None, scripts=None):
    if styles is not None and scripts is not None:
        return dict(styles=styles, scripts=scripts)
    return dict(
        styles=(static('hrendjango/css/' + css for css in STYLES)),
        scripts=(static('hrendjango/js/' + js for js in SCRIPTS))
    )



from hrendjango.management.base import BaseDNSCommand


class Command(BaseDNSCommand):
    action = 'add'
    help = "Добавляет название проекта в файл hosts"

    def add_arguments(self, parser):
        parser.add_argument('top_level_domain', type=str, help="Домен верхнего уровня", nargs='?', default=None)
        super().add_arguments(parser)



import platform


def windows_only(func):
    def wrapper(self, *args, **options):
        if platform.system() == 'Windows':
            return func(self, *args, **options)
        print("Данная функция работает только на Windows")
    return wrapper



from django import template


class Library(template.Library):
    def empty_tag(self, takes_context=None, name=None):
        def decorator(func):
            def wrapper(*args, **kwargs):
                func(*args, **kwargs)
                return ''
            return self.simple_tag(wrapper, takes_context=takes_context, name=name)
        return decorator

    # def block_tag(self, node_class, name=None):
    #     def decorator(func):
    #         def wrapper(parser, token):
    #             if func.__name__[:3] != 'do_':
    #                 raise NameError
    #             name_ = func.__name__[3:] if name is None else name

    def raise_block_tag_with_as_error(self, name):
        raise template.TemplateSyntaxError("Invalid syntax for '{name}'. Use {open} {name} {close} or {open} {name} "
                                           "as variable {close}".format(name=name, open='{%', close='%}'))


class Node(template.Node):
    pass



from typing import Optional
from .mixins import DebugRequiredMixin


class BaseDebug(DebugRequiredMixin):
    do_method_name: Optional[str] = None

    def do(self, *args, **kwargs):
        if self.do_method_name is not None:
            eval(f'self.{self.do_method_name}(*args, **kwargs)')
        else:
            raise ValueError(f'do_method_name not defined')




from django.conf import settings
from django.contrib import admin
from .models import get_file_model, get_image_model


if getattr(settings, 'HRENDJANGO_ACTIVE_MODELS', None):
    admin.site.register(get_file_model())
    admin.site.register(get_image_model())



import importlib
from django.conf import settings
from django.http import HttpResponse
from django.utils.deprecation import MiddlewareMixin
from .constants import URLS_ROOT
from .exceptions import Http401

handler401 = getattr(URLS_ROOT, 'handler401', None)
handler405 = getattr(URLS_ROOT, 'handler405', None)


class HrendjangoMiddleware(MiddlewareMixin):
    def process_exception(self, request, exception):
        if isinstance(exception, Http401):
            if handler401:
                return handler401(request)
            return HttpResponse("<h1>401 Unauthorized</h1>", status=401)

    def process_request(self, request):
        request.user_ip = request.META['REMOTE_ADDR']
        request.user_user_agent = request.META['HTTP_USER_AGENT']

    def process_response(self, request, response):
        if response.get('Content-Type', '').startswith('text/html'):
            favicon_url = '/static/favicon.png'  # Укажите путь к вашему favicon
            favicon_tag = f'    <link rel="icon" href="{favicon_url}">'
            response.content = response.content.replace(b'</head>', f'{favicon_tag}</head>'.encode('utf-8'))
        return response



import os
from django.db import models

__all__ = ['delete_files_on_delete']


def delete_files_on_delete(cls):
    """Декоратор класса для удаления файлов при удалении записи"""
    original_delete = cls.delete

    def new_delete(self, *args, **kwargs):
        for field in self._meta.get_fields():
            if isinstance(field, (models.FileField, models.ImageField)):
                file = getattr(self, field.name)
                if file and os.path.isfile(file.path):
                    os.remove(file.path)
        original_delete(self, *args, **kwargs)

    cls.delete = new_delete
    return cls



from django.http import Http404
from django.core.exceptions import *


class Http400(SuspiciousOperation):
    pass


class Http401(Exception):
    pass


class Http403(PermissionDenied):
    pass


class Http405(Exception):
    pass


class HttpLog405(Exception):
    pass



from typing import Optional
from deep_translator import GoogleTranslator as Translator
from django import template
from django.template import Library, Node
from django.conf import settings
from ..utils import translate_text as trans

register = Library()


@register.simple_tag
def translate_text(text, output_lang: Optional[str] = None, input_lang: Optional[str] = None):
    if input_lang is None:
        input_lang = 'en' if getattr(settings, 'HRENDJANGO_ENGLISH_INPUT_LANG', False) else 'auto'
    return trans(text, output_lang, input_lang)


@register.tag(name='translateblock')
def do_translateblock(parser, token):
    """
    Шаблонный тег для перевода текста через Google Translate.
    Синтаксис: {% translateblock 'source_lang' 'target_lang' %}текст{% endtranslateblock %}
    Пример: {% translateblock 'en' 'ru' %}Hello{% endtranslateblock %} → Привет
    """
    try:
        tag_name, input_lang, output_lang = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires exactly two arguments: source and target language." % token.contents.split()[0]
        )

    nodelist = parser.parse(('endtranslateblock',))
    parser.delete_first_token()
    return TranslateBlockNode(nodelist, input_lang, output_lang)


class TranslateBlockNode(template.Node):
    def __init__(self, nodelist, input_lang, output_lang):
        self.nodelist = nodelist
        self.input_lang = input_lang.strip("'\"")
        self.output_lang = output_lang.strip("'\"")

    def render(self, context):
        text = self.nodelist.render(context).strip()
        translator = Translator()
        try:
            translated = translator.translate(
                text,
                src=self.input_lang,
                dest=self.output_lang
            )
            return translated.text
        except Exception as e:
            return f"Translation error: {str(e)}"



from typing import Union
from django.urls.converters import StringConverter


class BaseLiteralStringConverter(StringConverter):
    allowed: Union[list, tuple, set]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.allowed:
            raise ValueError("Список разрешенных значений пуст")
        self.regex = '|'.join(self.allowed)

    def to_python(self, value):
        if value not in self.allowed:
            raise ValueError(f"Недопустимое значение. Допустимые: {self.allowed}")
        return value

    def to_url(self, value):
        if value not in self.allowed:
            raise ValueError(f"Недопустимое значение для URL. Допустимые: {self.allowed}")
        return value



from typing import Literal
from django.conf import settings
from django.core.management import BaseCommand
from hrenpack.listwork import dict_get
from hrenpack.cmd import is_admin
from hrenpack.system import HostsFile
from .decorators import windows_only


class BaseDNSCommand(BaseCommand):
    action: Literal['add', 'remove']
    hosts_file = HostsFile()

    def add_arguments(self, parser):
        parser.add_argument('-b', '--backup', action='store_true', help="Сделать резервную копию файла hosts")

    @windows_only
    def handle(self, *args, **options):
        if is_admin():
            top_level_domain = dict_get(options, 'top_level_domain', 'com')
            dont_backup = not options['backup']
            project_name = settings.ROOT_URLCONF.split('.')[0]
            domain_name = f'{project_name.lower()}.{top_level_domain}'
            if self.action == 'add':
                self.hosts_file.add_host('127.0.0.1', domain_name, dont_backup)
            else:
                self.hosts_file.remove_host(domain_name, dont_backup)
            print(f"{'Добавление' if self.action == 'add' else 'Удаление'} домена из файла hosts завершено успешно")
        else:
            print("Отказано в доступе")



from django.apps import AppConfig


class HrendjangoConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'hrendjango'
    verbose_name = 'Модели HrenDjango'
    default_settings = dict(
        HRENDJANGO_ACTIVE_MODELS='all',
        HRENDJANGO_FILE_MODEL='hrendjango.MediaFile',
        HRENDJANGO_IMAGE_MODEL='hrendjango.ImageMediaFile'
    )

    def ready(self):
        from django.conf import settings
        for key, value in self.default_settings.items():
            setattr(settings, key, getattr(settings, key, value))



import os, glob
from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.conf import settings
from hrenpack.date_and_time_work import now_to_str
from hrenpack.filework import TextFile
from hrendjango import __version__


class Command(BaseCommand):
    help = 'Creates a migration for the specified app and optionally runs it'
    app_label = 'hrendjango'
    migration_directory = os.path.join(settings.BASE_DIR, app_label, "migrations")
    file = None

    def add_arguments(self, parser):
        parser.add_argument('-m', '--migrate', action='store_true', help='Run the migration after creating it')

    def get_last_migration_filename(self):
        try:
            migration_files = glob.glob(os.path.join(self.migration_directory, '*.py'))
            migration_files = [f for f in migration_files if not f.endswith('__init__.py')]
            latest_migration = max(migration_files, key=os.path.getctime)
            return os.path.basename(latest_migration)
        except ValueError:
            return None

    def handle(self, *args, **options):
        migrate = options['migrate']
        latest_migration_filename = self.get_last_migration_filename()
        call_command('makemigrations', self.app_label)
        new_migration_filename = self.get_last_migration_filename()
        if latest_migration_filename != new_migration_filename:
            self.change_file(new_migration_filename)
        if migrate:
            call_command('migrate', self.app_label)

    def change_file(self, migration_name):
        self.file = TextFile(os.path.join(self.migration_directory, migration_name))
        split = '\n\nclass Migration(migrations.Migration):\n'
        pre_class_text, class_text = self.file.read().split(split)
        pre_class_text = (f'# Сгенерировано HrenDjango версии {__version__} {now_to_str()}\n' +
                          '\n'.join(pre_class_text.split('\n')[1:]))
        if 'migrations.CreateModel' in class_text:
            pre_class_text += 'from hrendjango.utils import create_model\n'
        pre_class_text += 'from hrenpack.listwork import del_none\n\n\n'
        class_text = split + class_text
        class_text = class_text.replace('operations = [', 'operations = del_none(')
        class_text = class_text.replace('migrations.CreateModel', 'create_model')
        class_text = class_text[:-2] + ')\n'
        self.file.rewrite(pre_class_text + class_text[2:])



from typing import Literal, Callable
from functools import wraps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, JsonResponse
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import render_to_string
from django.views.decorators.http import require_http_methods as django_require_http_methods
from .exceptions import Http401


def require_http_methods(*methods: Literal['GET', 'POST', 'PUT', 'DELETE'], error_mode: bool = True) -> Callable:
    def decorator(view_function):
        @wraps(view_function)
        def if_error(request, *args, **kwargs):
            return django_require_http_methods(methods)(view_function)(request, *args, **kwargs)

        @wraps(view_function)
        def not_error(request, *args, **kwargs):
            pass

        return if_error if error_mode else not_error


def debug_required(func):
    def wrapper(*args, **kwargs):
        if not settings.DEBUG:
            raise ImproperlyConfigured('Only debug mode is on')
        return func(*args, **kwargs)
    return wrapper



from django.http import HttpResponse, HttpResponseNotAllowed, HttpResponseGone
from django.shortcuts import render as django_render


def render_construct(code: int):
    def render(request, template_name: str, context: dict, content_type=None, using=None):
        return django_render(request=request, template_name=template_name, context=context,
                             content_type=content_type, status=code, using=using)

    # return lambda request, template_name, context, content_type=None, using=None: django_render(
    #     request=request, template_name=template_name, context=context, content_type=content_type,
    #     status=code, using=using)
    return render


render_permanent_redirect = render_construct(301)
render_redirect = render_construct(302)
render_not_modified = render_construct(304)
render_bad_request = render_construct(400)
render_forbidden = render_construct(403)
render_not_found = render_construct(404)
render_gone = render_construct(410)
render_server_error = render_construct(500)
render_internal_server_error = render_construct(501)
render_not_implemented = render_construct(502)
render_bad_gate = render_construct(503)



import platform, subprocess, requests, os
from django.core.management.base import BaseCommand
from hrenpack.network import download_file_to_temp
from hrendjango.management.decorators import windows_only


class Command(BaseCommand):
    help = "Установка Node.js. Используйте 'version' для указания версии, или '-b' для установки последней бета-версии."

    def add_arguments(self, parser):
        parser.add_argument('version', type=str, help="Версия Node.js. Преобладает над аргументом --use-beta",
                            nargs='?', default=None)
        parser.add_argument('-b', '--use-beta', action='store_true',
                            help="Использование последней доступной версии Node.js. "
                                 "Если не указан, используется последняя стабильная версия")

    @staticmethod
    def get_installed_version():
        try:
            result = subprocess.run(["node", "-v"], capture_output=True, text=True, check=True)
            return result.stdout.strip()
        except (subprocess.CalledProcessError, FileNotFoundError):
            return None

    @staticmethod
    def get_latest_version(use_beta):
        versions = requests.get('https://nodejs.org/dist/index.json').json()
        for version in versions:
            if version['lts'] or use_beta:
                return version['version']

    @windows_only
    def handle(self, *args, **options):
        installed_version = self.get_installed_version()
        if installed_version is not None:
            print(f"Node.js уже установлен, версия: {installed_version}. Если хотите переустановить его,",
                  "то сначала удалите старую версию")
        if options['version']:
            version = options['version']
            if 'v' not in version:
                version = 'v' + version
        else:
            version = self.get_latest_version(options['use_beta'])
        url = f'https://nodejs.org/dist/{version}/node-{version}-x64.msi'
        print("Скачивание Node.js...")
        # path = download_file_to_temp(url, 'msi', use_progressbar=True)
        path = 'C:/node-v22.14.0-x64.msi'
        if os.path.exists(path) and os.path.getsize(path) > 0:
            print("Файл успешно загружен.")
        else:
            print("Ошибка при загрузке файла.")
        try:
            print("Установка Node.js...")
            os.system(' '.join([path, 'INSTALLDIR="C:/Program Files/nodejs/"', '/quiet', '/norestart']))
            print("Node.js успешно установлен.")
        except subprocess.CalledProcessError as e:
            print(f"Ошибка при установке Node.js: {e}")



from hrenpack.python import import_all_submodules

"""
hrenpack_dir = Path(__file__).parent

for module in hrenpack_dir.glob('*.py'):
    if module.name == '__init__':
        continue
    import_module(f'.{module.stem}', package=__package__)
"""

import_all_submodules(__name__)



from pip_setuptools import setup, find_packages, requirements, clean

clean()
setup(
    name='hrendjango',
    version='0.1.0',
    packages=find_packages(),
    include_package_data=True,
    author="Маг Ильяс DOMA (MagIlyasDOMA)",
    author_email='magilyas.doma.09@list.ru',
    python_requires='>=3.10',
    install_requires=requirements(),
    description='Global add-on to Django',
    license='MIT',
    url='https://github.com/MagIlyasDOMA/hrendjango',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Programming Language :: Python :: 3 :: Only',
    ]
)



from django.core.management.base import BaseCommand
from django.utils.translation import gettext


class Command(BaseCommand):
    def handle(self, *args, **options):
        while True:
            print(gettext(input(gettext("Enter the expression to translate") + ':\n')))



from .base import AbstractFile, AbstractImageFile

__all__ = ['MediaFile', 'ImageMediaFile']


class MediaFile(AbstractFile):
    class Meta(AbstractFile.Meta):
        abstract = False
        swappable = 'HRENDJANGO_FILE_MODEL'


class ImageMediaFile(AbstractImageFile):
    class Meta(AbstractImageFile.Meta):
        abstract = False
        swappable = 'HRENDJANGO_IMAGE_MODEL'



import os
from django.db import models
from django.core.files.storage import default_storage

__all__ = ['DeleteFilesMixin']


class DeleteFilesMixin:
    """Миксин для гарантированного удаления файлов при удалении модели"""

    def delete(self, *args, **kwargs):
        # Получаем полную информацию о файлах ДО удаления модели
        files_info = self._collect_files_info()

        # Удаляем модель (включая все связанные объекты)
        result = super().delete(*args, **kwargs)

        # Удаляем файлы после успешного удаления модели
        self._delete_collected_files(files_info)
        return result

    def _collect_files_info(self):
        """Собирает информацию о всех файлах модели"""
        files_info = []
        for field in self._meta.get_fields():
            if isinstance(field, (models.FileField, models.ImageField)):
                file = getattr(self, field.name)
                if file:
                    files_info.append({
                        'field': field.name,
                        'file': file,
                        'path': file.path if hasattr(file, 'path') else None,
                        'name': file.name,
                        'storage': file.storage
                    })
        return files_info

    def _delete_collected_files(self, files_info):
        """Удаляет собранные файлы с обработкой всех исключений"""
        for info in files_info:
            try:
                storage = info['storage']
                name = info['name']

                # Удаляем основной файл
                if storage.exists(name):
                    storage.delete(name)

                # Для ImageField удаляем все вариации (thumbnails)
                if hasattr(info['file'], 'field') and isinstance(info['file'].field, models.ImageField):
                    self._delete_image_variations(info)
            except Exception as e:
                print(f"Failed to delete file {name}: {str(e)}")

    def _delete_image_variations(self, file_info):
        """Удаляет вариации изображений (для ImageField)"""
        try:
            field = file_info['file'].field
            if hasattr(field, 'variations'):
                for variation in field.variations:
                    variation_name = file_info['file'].field.get_variation_name(
                        file_info['name'], variation
                    )
                    if file_info['storage'].exists(variation_name):
                        file_info['storage'].delete(variation_name)
        except Exception as e:
            print(f"Failed to delete image variations: {str(e)}")

    @classmethod
    def register_signals(cls):
        """Регистрирует сигналы для обработки bulk/cascade удаления"""
        from django.db.models.signals import pre_delete
        from django.dispatch import receiver

        @receiver(pre_delete, sender=cls)
        def handle_pre_delete(sender, instance, **kwargs):
            if isinstance(instance, DeleteFilesMixin):
                files_info = instance._collect_files_info()
                instance._delete_collected_files(files_info)



from typing import Optional
from django.http import HttpRequest
from django.template import Library
from django.utils.safestring import mark_safe

register = Library()


@register.inclusion_tag('hrendjango/includes/form_context.html', takes_context=True)
def form_context(context, button_text: Optional[str] = None, button_class: Optional[str] = None, div_mode: bool = False,
                 br_post: bool = True, colon_post: bool = True, form=None, mode: str = 'normal', **kwargs):
    if form is None:
        form = context['form']
    if button_class is None:
        button_class = 'btn btn-hren'
    return {'form': form, 'button_text': button_text, 'br_post': br_post, 'fc_mode': mode,
            'colon_post': colon_post, 'button_class': button_class, 'div_mode': div_mode, **kwargs}


@register.simple_tag
def get_condition(condition: bool, true='', false=''):
    return true if condition else false


@register.simple_tag
def user_full_name(user):
    return user.get_full_name()


@register.simple_tag(takes_context=True)
def current_user_full_name(context):
    user = context['user']
    if user.is_authenticated:
        return user_full_name(user)


@register.inclusion_tag('system/includes/account.html', takes_context=True)
def account_content(context):
    context['name'] = current_user_full_name(context)
    return context


@register.simple_tag
def crutch():
    return mark_safe('<p style="color: transparent">.</p>')


@register.simple_tag
def html_text(text: str):
    return mark_safe(text)


@register.simple_tag
def simple_text(text: str):
    return text


@register.simple_tag
def var(arg):
    return arg


@register.simple_tag(takes_context=True)
def current_page(context):
    return context['request'].path


"""
@register.tag(name='varblock')
def do_varblock(parser, token):
    bits = token.split_contents()
    if len(bits) == 4 and bits[2] == 'as':
        # Если используется синтаксис {% mytag as variable %}
        nodelist = parser.parse(('endvarblock',))
        parser.delete_first_token()  # Удаляем {% endmytag %} из потока
        return VarBlockNode(nodelist, bits[3])
    elif len(bits) == 1:
        # Если используется обычный синтаксис {% mytag %}
        nodelist = parser.parse(('endvarblock',))
        parser.delete_first_token()
        return VarBlockNode(nodelist)
    else:
        register.raise_block_tag_with_as_error('varblock')


class VarBlockNode(Node):
    def __init__(self, nodelist, variable_name=None):
        self.nodelist = nodelist
        self.variable_name = variable_name

    def render(self, context):
        output = self.nodelist.render(context)
        if self.variable_name:
            context[self.variable_name] = output
            return ''  # Возвращаем пустую строку, так как результат сохранён в переменную
        else:
            return output
"""



# Generated by Django 5.1.6 on 2025-04-15 14:00

import hrendjango.models.mixins
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ImageMediaFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='Уникальный идентификатор')),
                ('date_create', models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')),
                ('date_update', models.DateTimeField(auto_now=True, verbose_name='Дата изменения')),
                ('filename', models.CharField(max_length=255, verbose_name='Имя файла')),
                ('file', models.ImageField(upload_to='images/', verbose_name='Изображение')),
            ],
            options={
                'verbose_name': 'изображение',
                'verbose_name_plural': 'Изображения',
                'abstract': False,
                'swappable': 'HRENDJANGO_IMAGE_MODEL',
            },
            bases=(hrendjango.models.mixins.DeleteFilesMixin, models.Model),
        ),
        migrations.CreateModel(
            name='MediaFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='files/', verbose_name='Файл')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='Уникальный идентификатор')),
                ('date_create', models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')),
                ('date_update', models.DateTimeField(auto_now=True, verbose_name='Дата изменения')),
                ('filename', models.CharField(max_length=255, verbose_name='Имя файла')),
            ],
            options={
                'verbose_name': 'файл',
                'verbose_name_plural': 'Файлы',
                'abstract': False,
                'swappable': 'HRENDJANGO_FILE_MODEL',
            },
            bases=(hrendjango.models.mixins.DeleteFilesMixin, models.Model),
        ),
    ]



from django.conf import settings
from hrenpack.framework.django import get_view_base_template


# def base_template_context(request):
#     view = request.resolver_match.func
#     view = getattr(view, 'view_class', view)
#     return dict(base_template=get_view_base_template(view))


# def base_template_context(request):
#     try:
#         view = request.resolver_match.func
#         view = getattr(view, 'view_class', view)
#         base_template = get_view_base_template(view)
#         if not base_template:
#             raise ValueError("Empty template name")
#         return {'base_template': base_template}
#     except Exception as e:
#         # Логируйте ошибку для отладки
#         import logging
#         logging.getLogger(__name__).error(f"Error in base_template_context: {e}")
#         return {'base_template': 'empty.html'}  # Значение по умолчанию


def protocol_and_domain(request):
    return dict(protocol='https' if request.is_secure() else 'http', domain=request.get_host())
